#include <stdio.h>
#include <cuda.h>
#include <cuda_runtime_api.h>
#include <sys/time.h>
#include <assert.h>

#define NITER	7500

static cudaError_t status;
static unsigned iter;

static struct timeval start;
static struct timeval end;

int main(int argc, char **argv)
{
	/* Initialize CUDA */
	status = cudaFree(0);
	assert(!status);

	cudaStream_t stream;
	status = cudaStreamCreate(&stream);
	assert(!status);

	float *h_array;
	cudaMallocHost((void **)&h_array, 5*sizeof(double));
	assert(h_array);

	float *d_array;
	cudaMalloc((void **)&d_array, 5*sizeof(double));
	assert(d_array);

	gettimeofday(&start, NULL);

	for (iter = 0; iter < NITER; iter++)
	{
		cudaMemcpyAsync(d_array, h_array, 5*sizeof(double), cudaMemcpyHostToDevice, 0);
		//status = cudaThreadSynchronize();	
		status = cudaStreamSynchronize(0);
		assert(!status);
	}
	
	gettimeofday(&end, NULL);

	double timing0 = (double)((end.tv_sec - start.tv_sec)*1000000ULL + (end.tv_usec - start.tv_usec));

	fprintf(stdout, "Using stream 0:\n");
	fprintf(stdout, "TOTAL %2.2f ms\n", timing0/1000);
	fprintf(stdout, "Per iteration: %2.2f us\n", timing0/NITER);

	gettimeofday(&start, NULL);

	for (iter = 0; iter < NITER; iter++)
	{
		cudaMemcpyAsync(d_array, h_array, 5*sizeof(double), cudaMemcpyHostToDevice, stream);
		status = cudaStreamSynchronize(stream);
		assert(!status);
	}
	
	gettimeofday(&end, NULL);

	double timing1 = (double)((end.tv_sec - start.tv_sec)*1000000ULL + (end.tv_usec - start.tv_usec));

	fprintf(stdout, "Using stream from application:\n");
	fprintf(stdout, "TOTAL %2.2f ms\n", timing1/1000);
	fprintf(stdout, "Per iteration: %2.2f us\n", timing1/NITER);


	return 0;
}
